SUBROUTINE solarcalc(slope,aspect,day,solaz,solel,skyview,terrainview,shade_cell,P,G,potential_atm,gout)
USE params
USE nrtype
!USE OMP_LIB
IMPLICIT NONE

!***********************************************************************
!c Solar radiation notes:								c
!c   Because this model is set up to run with varying timesteps, the   c
!c   treatment of solar radiation, specifically shortwave radiation,   c
!c   needs some special attention.  For daily timesteps the model will c
!c   intrinsically underestimate incoming shortwave rad (inswrad)	c
!c   based on the assumption that the inswrad value supplied is a	c
!c   daily average.  Also, a daily average value cannot be adjusted	c
!c   for slope and aspect at the glacier surface.				c
!c   So, daily timesteps are treated as follows:				 c
!c											   c
!c The model calculates clear sky direct solar radiation for daylight  c
!c	hours at each grid node using solar elevation and azimuth data   c
!c	from the "solar" subroutine using the following calculations:	c
!c  - The optical air mass is calculated from Robinson (1966),	   c
!c	(eqn. 3.12) with a correction applied for elevation by multi-  c
!c	plying the air mass by the ratio of local pressure to sea-	 c
!c	level pressure (Oke, 1987).						c
!c  - An air transmissivity of 0.84 was used. (Oke, 1987)		  c
!c  - Radius vector of the earth's orbit Williams et.al., 1972)(p. 527)c
!c  - Insolation on a horizontal surface from Williams et. al., 1972)  c
!c	(eqn. 1a)									c
!c  Note:  these equations give only a 1st order approximation of	c
!c	solar radiation, but they are used as a max. value and should be c
!c	valid as such.								   c
!c											   c
!c  After each hourly value is calculated for a horizontal surface,	c
!c	the value is adjusted for the slope and aspect of the glacier	c
!c	surface following Arnold et.al., 1996 (eqn. 7).			c
!c											   c
!c The model averages this clear sky approximation for the day, and	c
!c	divides this average by the average supplied in the met file to  c
!c	obtain a ratio of theoretical to observed solar radiation.	 c
!c The ratio is then applied to the hourly values of theor. radiation  c
!c	to reduce them proportionatly to the observed data, producing	c
!c	a daily total of inswrad to be applied to the Eflux!		 c
!c											   c
!c Following Arnold et.al., 1996, the diffuse radiation is assumed to  c
!c	be 20 percent of the direct, and is applied via a view factor	c
!c	relationship to the glacier surface (Oke, 1987).			 c
!***********************************************************************

	REAL(DP),							INTENT(IN   )	:: day
	REAL(DP),   DIMENSION(1:24),		INTENT(IN   )	:: solaz, &
														   solel

	REAL(DP),							INTENT(IN   )	:: P, &
														   skyview, &
														   terrainview, &
														   slope, &
														   aspect

	REAL(DP),   DIMENSION(1:36),		INTENT(IN   )	:: shade_cell

	REAL(DP),							INTENT(IN   )	:: G
	REAL(DP),							INTENT(  OUT)	:: potential_atm,gout

	REAL(DP),							DIMENSION(1:24)	:: dirs, &
											   slope_asps

	REAL(DP)											:: rvec=0., &
														   ins=0., &
								   						   dif=0., &
								   						   totg=0., &
								   						   tots=0., &
								   						   saz=0., &
								   						   sel=0., &
								   						   azim=0., &
								   						   m=0., &
								   						   costheta, &
								   						   rad_ratio=0., &
								   						   direct_meas=0., &
								   						   diffuse_rad=0., &
								   						   direct_rad=0., &
														   dir_frac=0., &
														   tot_atm=0., &
														   max_atm=0., &
														   costhetas=0., &
														   sunnyhours=0._dp, &
														   diffuse_frac





	INTEGER(I4B)										:: j,round
	dir_frac=0._dp
	potential_atm=0._dp
	tot_atm=0._dp
	totg=0._dp
	tots=0._dp
	slope_asps(:)=0._dp
	sunnyhours=0._dp
	Rvec=eccentricity*COS(precession+(79._dp-day)*(360._dp*pi/(365._dp*180._dp)))
	IF (tmstp .EQ. 86400) THEN		! Must simulate the artificial solar cycle
!	Inserting a short circuit for the laurentide simulations. This is probably okay as a start
!	as there are no shading issues, and slopes and aspects are minor, so having solar unadjusted by
!	diffuse and direct parsing shouldn't be a big deal.

!	So, this will simply adjust the solar incoming by slope and aspect and pass it back. Hope this fixes things!

		max_atm=solconst*((1-rvec)**2)*(trans**(P/(101325*SIN(MAXVAL(solel(:))))))*SIN(MAXVAL(solel(:)))
		DO j=1,24			!FSA:	Begin loop to artificially cycle thgrough hourly solar rad.
			Saz=solaz(j)
			Sel=solel(j)
			IF (Sel.gt.0.) THEN
				IF (sel .LT. 0.17) THEN		!Want to eliminate the low sun problem!!!
					sel=0.17_dp
				ENDIF
				sunnyhours=sunnyhours+1._dp
				azim=round(Saz/rad/10._dp+13._dp)
!c compute optical airmass
				m=(SQRT((r_h*SIN(Sel))**2+2*r_h+1)-r_h*sin(Sel))*(P/101300.)
!c compute incoming direct rad for hoirzontal surface
				Ins=solconst*((1-rvec)**2)*sin(Sel)*0.84**m		!FSA:	The Solconst/Rvec*Rvec may be an issue
!c adjust for slope and aspect and shading
				potential_atm=solconst*((1-rvec)**2)*(trans**(P/(101325*SIN(sel))))*SIN(sel)

				!WRITE(UNIT=*,FMT='(5(g10.2,1x))') day,REAL(j),potential_atm,saz,sel
				tot_atm=tot_atm+potential_atm
				IF (Sel .GT. shade_cell(round(azim))) THEN
					DirS(j)=Ins*(costheta(slope,aspect,(Saz+pi),Sel))
					dir_frac=dir_frac+(potential_atm/max_atm)*(costheta(slope,aspect,Saz,Sel))
					slope_asps(j)=costheta(slope,aspect,Saz,Sel)
				ELSE
					DirS(j)=0._dp
					dir_frac=dir_frac+(potential_atm/max_atm)
				ENDIF
!c calculate diffuse rad.
				Dif=0.2*ins*skyview+terrain_albedo*ins*terrainview
				TotG=TotG+Ins+dif
				TotS=TotS+DirS(j)+Dif
			ELSE
				dir_frac=dir_frac+0._dp
				DirS(j)=0.0
			ENDIF
		ENDDO
		IF (tot_atm .EQ. 0._dp) THEN
			rad_ratio=1._dp
			sunnyhours=1._dp
			gout=0._dp
			RETURN
		ELSE
			rad_ratio=G/(tot_atm/24._dp)
		ENDIF
		IF (rad_ratio .GE. maxcloudthresh) THEN	!No clouds, cell is under direct sunlight
			diffuse_rad=0.2_dp*G
			direct_rad=0.8_dp*G
		ELSEIF ((rad_ratio .LT. maxcloudthresh) .AND. (rad_ratio .GT. mincloudthresh)) THEN !Mix of sun and cloud
			diffuse_rad=(-2._dp*rad_ratio+1.8_dp)*G
			direct_rad=(1._dp-(-2._dp*rad_ratio+1.8_dp))*G
		ELSE
			diffuse_rad=G
			direct_rad=0._dp
		ENDIF
		IF (sunnyhours .EQ. 0.0_dp) sunnyhours=1.0_dp
		gout=diffuse_rad*(skyview)+direct_rad*(terrain_albedo*terrainview+(SUM(slope_asps(:))/sunnyhours))
	ELSE
		IF (sw_stat) THEN	!There is an incoming SW record
			Sel=solel(1)
			IF (Sel .GT. 0._dp) THEN
				Saz=solaz(1)
				costhetas=costheta(slope,aspect,(-saz),sel)
				azim=((-Saz/rad+180)/10)
				direct_meas=G
				potential_atm=solconst*((1-rvec)**2)*(trans**(P/(101325*SIN(sel))))*costheta(0._dp,0._dp,(-saz),sel)
				rad_ratio=direct_meas/potential_atm
				IF (rad_ratio .GE. maxcloudthresh) THEN	!No clouds, cell is under direct sunlight
					diffuse_frac=0.2_dp
				ELSEIF ((rad_ratio .LT. maxcloudthresh) .AND. (rad_ratio .GT. mincloudthresh)) THEN !Mix of sun and cloud
					diffuse_frac=(-2._dp*rad_ratio+1.8_dp)
				ELSE
					diffuse_frac=1._dp
				ENDIF
				diffuse_rad=diffuse_frac*G
				direct_rad=(1._dp-diffuse_frac)*G
				IF (Sel .LT. shade_cell(round(azim))) THEN	!The sun is down
					gout=diffuse_rad*(skyview)
					potential_atm=0._dp
				ELSE		!The sun is up
					gout=diffuse_rad*(skyview)+direct_rad*terrain_albedo*terrainview+skyview*direct_rad/SIN(sel)*costhetas
					potential_atm=potential_atm*costhetas
				ENDIF
			ELSE
				gout=0._dp
			END IF
		ELSE
			CONTINUE
		ENDIF
	ENDIF
END SUBROUTINE solarcalc
